from __future__ import division, with_statement

__version__ = "$Revision: 320 $"

"""
Assorted utility functions and classes common or useful to most of Segtools.


Author: Orion Buske <stasis@uw.edu>
"""

# XXX: the things in here should be moved to __init__; test

import os
import sys

from contextlib import closing, contextmanager
from functools import partial
from gzip import open as _gzip_open
from warnings import warn
from time import time

from numpy import array, empty, iinfo, ndarray, searchsorted
from tabdelim import DictReader, DictWriter, ListReader, ListWriter

from . import log, EXT_GZ, PKG, die

PKG_RESOURCE = os.extsep.join([PKG, "resources"])

EXT_PDF = "pdf"
EXT_PNG = "png"
EXT_TAB = "tab"
EXT_DOT = "dot"
EXT_HTML = "html"
EXT_DIV = "div"
EXT_SLIDE = os.extsep.join(["slide", EXT_PNG])
EXT_THUMB = os.extsep.join(["thumb", EXT_PNG])  # for thumbnails
EXT_SUMMARY = os.extsep.join(["summary", EXT_TAB])

IMG_EXTS = [EXT_PNG, EXT_PDF, EXT_SLIDE, EXT_THUMB]
NICE_EXTS = dict(tab=EXT_TAB, pdf=EXT_PDF, png=EXT_PNG, html=EXT_HTML,
                 div=EXT_DIV, slide=EXT_SLIDE, thumb=EXT_THUMB,
                 summary=EXT_SUMMARY, dot=EXT_DOT)

SUFFIX_GZ = os.extsep + EXT_GZ

LABEL_ALL = "all"

THUMB_SIZE = 100

class OutputMasker(object):
    """Class to mask the output of a stream.

    Suggested usage:
      sys.stout = OutputMasker(sys.stdout)  # Start masking
      <commands with stout masked>  # Masked commands
      sys.stdout = sys.stdout.restore()  # Stop masking
    """
    def __init__(self, stream=None):
        self._stream = stream
    def write(self, string):
        pass  # Mask output
    def writelines(self, lines):
        pass  # Mask output
    def restore(self):
        return self._stream

class ProgressBar(object):
    def __init__(self, total=67, width=67, chr_done="#", chr_undone="-",
                 format_str="Progress: [%(done)s%(undone)s]",
                 out=sys.stdout):
        """Create a progress bar for num_items that is width characters wide

        total: the number of items in the task (calls to next before 100%)
        width: the width of the bar itself, not including labels
        chr_done: character for displaying completed work
        chr_undone: character for displaying uncompleted work
        format_str: format string for displaying progress bar. Must contain
          a reference to %(done)s and %(undone)s, which will be substituted
        out: an object that supports calls to write() and flush()
        """
        assert int(width) > 0
        assert int(total) > 0

        self._width = int(width)
        self._n = int(total)
        self._quantum = float(self._n / self._width)
        self._chr_done = str(chr_done)
        self._chr_undone = str(chr_undone)
        self._format_str = str(format_str)
        self._out = out

        self._i = 0
        self._progress = 0
        self.refresh()

    def next(self):
        """Advance to the next item in the task

        Might or might not refresh the progress bar
        """
        self._i += 1
        if self._i > self._n:
            raise StopIteration("End of progress bar reached.")

        progress = int(self._i / self._quantum)
        if progress != self._progress:
            self._progress = progress
            self.refresh()

    def refresh(self):
        """Refresh the progress bar display"""
        fields = {"done": self._chr_done * self._progress,
                  "undone": self._chr_undone * (self._width - self._progress)}
        self._out.write("\r%s" % (self._format_str % fields))
        self._out.flush()

    def end(self):
        """Complete the job progress, regardless of current state"""
        self._progress = self._width
        self.refresh()
        self._out.write("\n")
        self._out.flush()


class FeatureScanner(object):
    def __init__(self, features):
        self._current = None
        self._nearest = None
        if isinstance(features, ndarray):
            self._iter = iter(features)
            try:
                self._current = self._iter.next()
            except StopIteration:
                pass
            else:
                self._nearest = self._current
        else:
            warn("Feature list not of understood type. Treated as empty")

        self._has_next = (self._current is not None)
        self._overlapping = []

        self._prev_best = None
        self._next_best = None
        self._prev_start = None

    @staticmethod
    def distance(feature1, feature2):
        """Return the distance between two features (zero if overlap)"""
        if feature1['end'] <= feature2['start']:
            return feature2['start'] - feature1['end'] + 1
        elif feature2['end'] <= feature1['start']:
            return feature1['start'] - feature2['end'] + 1
        else:  # Features overlap
            return 0

    @staticmethod
    def are_overlapping(feature1, feature2):
        """Return the whether or not two features overlap"""
        return feature1['start'] < feature2['end'] and \
                feature2['start'] < feature1['end']

    def _seek(self, segment):
        # Seek internal records up to segment, updated self._overlapping
        # and self._nearest

        # Clear out any non-overlapping features from self._overlapping
        if self._overlapping:
            i = 0
            while i < len(self._overlapping):
                if self.are_overlapping(segment, self._overlapping[i]):
                    i += 1
                else:
                    del self._overlapping[i]

        # If any are overlapping, nearest is 0 distance
        if self._overlapping:
            nearest_dist = 0
            self._nearest = self._overlapping[0]
        elif self._nearest is not None:
            # Closest feature will start as the closest from the last iteration
            nearest_dist = self.distance(self._nearest, segment)
        else:
            assert not self._has_next

        # Scan through features until we get past segment
        if self._has_next:
            segment_start = segment['start']
            segment_end = segment['end']
            while self._current['start'] < segment_end:
                # dist will be negative if this overlaps
                dist = segment_start - self._current['end'] + 1
                if dist < 0:
                    self._overlapping.append(self._current)

                if dist < nearest_dist:
                    self._nearest = self._current
                    nearest_dist = dist

                try:
                    self._current = self._iter.next()
                except StopIteration:
                    self._has_next = False
                    break

            # Check distance of first feature past segment as well
            dist = self._current['start'] - segment_end + 1
            if dist < nearest_dist:
                self._nearest = self._current
                nearest_dist = dist

    def nearest(self, segment):
        """Return a list of the nearest features for the next segment

        This will contain a single feature if no features overlap the given
        segment, else it will be a list of features that overlap.

        If no features were found at all, this will be None.

        Should be called with in-order segments

        """
        self._seek(segment)
        if self._overlapping:
            return self._overlapping
        elif self._nearest is None:
            return None
        else:
            return self._nearest


## UTILITY FUNCTIONS

def inverse_dict(d):
    """Given a dict, returns the inverse of the dict (val -> key)"""
    res = {}
    for k, v in d.iteritems():
        assert v not in res
        res[v] = k
    return res

def make_filename(dirpath, basename, ext):
    return os.path.join(dirpath, os.extsep.join([basename, ext]))

make_tabfilename = partial(make_filename, ext=EXT_TAB)
make_htmlfilename = partial(make_filename, ext=EXT_HTML)
make_divfilename = partial(make_filename, ext=EXT_DIV)
make_pngfilename = partial(make_filename, ext=EXT_PNG)
make_pdffilename = partial(make_filename, ext=EXT_PDF)
make_thumbfilename = partial(make_filename, ext=EXT_THUMB)
make_slidefilename = partial(make_filename, ext=EXT_SLIDE)
make_summaryfilename = partial(make_filename, ext=EXT_SUMMARY)
make_dotfilename = partial(make_filename, ext=EXT_DOT)

def make_namebase_summary(namebase):
    return os.extsep.join([namebase, "summary"])

def make_id(modulename, dirpath):
    return "_".join([modulename, os.path.basename(dirpath)])

def check_clobber(filename, clobber):
    if (not clobber) and os.path.isfile(filename):
        raise IOError("Output file: %s already exists."
                      " Use --clobber to overwrite." % filename)

def gzip_open(*args, **kwargs):
    return closing(_gzip_open(*args, **kwargs))

def maybe_gzip_open(filename, *args, **kwargs):
    if filename.endswith(SUFFIX_GZ):
        return gzip_open(filename, *args, **kwargs)
    else:
        return open(filename, *args, **kwargs)

def fill_array(scalar, shape, dtype=None, *args, **kwargs):
    if dtype is None:
        dtype = array(scalar).dtype

    res = empty(shape, dtype, *args, **kwargs)
    res.fill(scalar)
    return res

@contextmanager
def tab_saver(dirpath, basename, fieldnames=None, header=True,
              clobber=False, metadata=None, verbose=True):
    """Save data to tab file

    If fieldnames are specified, a DictWriter is yielded
    If fieldnames are not, a ListWriter is yielded instead

    metadata: a dict describing the data to include in the comment line.
      Comment line will start with '# ' and then will be a space-delimited
      list of <field>=<value> pairs, one for each element of the dict.
    """
    outfilename = make_tabfilename(dirpath, basename)
    log("Saving tab file: %s" % outfilename, verbose)
    start = time()
    check_clobber(outfilename, clobber)
    with open(outfilename, "w") as outfile:
        if metadata is not None:
            assert isinstance(metadata, dict)
            metadata_strs = ["%s=%s" % pair for pair in metadata.iteritems()]
            print >>outfile, "# %s" % " ".join(metadata_strs)

        if fieldnames:
            yield DictWriter(outfile, fieldnames, header=header,
                             extrasaction="ignore")
        else:
            yield ListWriter(outfile)

    log("Saved tab file in %.1f seconds" % (time() - start), verbose)

@contextmanager
def tab_reader(dirpath, basename, verbose=True, fieldnames=False):
    """Reads data from a tab file

    Yields a tuple (Reader, metadata)
    If fieldnames is True, a DictReader is yielded
    If fieldnames is False, a ListReader is yielded instead

    metadata: a dict describing the data included in the comment line.
    """

    infilename = make_tabfilename(dirpath, basename)
    log("Reading tab file: %s" % infilename, verbose)
    start = time()
    if not os.path.isfile(infilename):
        raise IOError("Unable to find tab file: %s" % infilename)

    with open(infilename, "rU") as infile:
        infile_start = infile.tell()
        comments = infile.readline().strip().split()
        metadata = {}
        if comments and comments[0] == "#":
            # Found comment line
            for comment in comments[1:]:
                name, value = comment.split("=")
                metadata[name] = value
        else:
            infile.seek(infile_start)

        if fieldnames:
            yield DictReader(infile), metadata
        else:
            yield ListReader(infile), metadata

    log("Tab file read in %.1f seconds" % (time() - start), verbose)

def map_segment_label(segments, range):
    """Flatten segments to a segment label per base vector

    Returns a tuple of:
      numpy.array of the segment key at every base
      sentinel value assigned to non-overlapped bases

    The given range must either be a tuple of (start, end), or
      have 'start' and 'end' attributes, and the returned array
      will represent the segment keys found between start and end.

    Thus, if start is 1000, and a segment with key 4 starts at 1000, the
    value of keys[0] is 4.

    """
    if isinstance(range, tuple):
        map_start, map_end = range
    else:
        map_start = range.start
        map_end = range.end

    map_size = map_end - map_start
    assert map_size >= 0

    # Choose sentinel value as maximum value supported by datatype
    segments_dtype = segments['key'].dtype
    sentinel = iinfo(segments_dtype).max
    assert sentinel != segments['key'].max()  # not already used

    res = fill_array(sentinel, (map_size,), segments_dtype)
    if map_size == 0:
        return res

    # will overwrite in overlapping case
    for segment in segments:
        start = segment['start']
        end = segment['end']
        key = segment['key']
        if start < map_end and end > map_start:
            start_i = max(start - map_start, 0)
            end_i = min(end - map_start, map_size)
            res[start_i:end_i] = key

    return res, sentinel

## Yields segment and the continuous corresponding to it, for each segment
##   in the chromosome inside of a supercontig
def iter_segments_continuous(chromosome, segmentation, verbose=True):
    chrom = chromosome.name
    try:
        segments = segmentation.chromosomes[chrom]
    except KeyError:
        raise StopIteration

    supercontig_iter = chromosome.itercontinuous()
    supercontig = None
    supercontig_last_start = 0
    nsegments = len(segments)

    if verbose:
        format_str = "".join([chrom, ":\t[%(done)s%(undone)s]"])
        progress = ProgressBar(nsegments, out=sys.stderr,
                               format_str=format_str)

    for segment in segments:
        start = segment['start']
        end = segment['end']

        while supercontig is None or start >= supercontig.end:
            try:
                # Raises StopIteration if out of supercontigs
                supercontig, continuous = supercontig_iter.next()
            except StopIteration:
                if verbose:
                    progress.end()
                raise

            # Enforce increasing supercontig indices
            assert supercontig.start >= supercontig_last_start
            supercontig_last_start = supercontig.start

        if verbose:
            progress.next()

        if end <= supercontig.start:
            continue  # Get next segment

        try:
            sc_start = supercontig.project(max(start, supercontig.start))
            sc_end = supercontig.project(min(end, supercontig.end))
            yield segment, continuous[sc_start:sc_end]
        except:
            for k, v in locals():
                print >>sys.stderr, "%r: %r" % (k, v)
            raise

    if verbose:
        progress.end()

## Yields supercontig and the subset of segments which overlap it.
def iter_supercontig_segments(chromosome, segmentation, verbose=True):
    try:
        segments = segmentation.chromosomes[chromosome.name]
    except KeyError:
        raise StopIteration

    for supercontig in chromosome:
        # Since it's a segmentation, there are no overlapping segments.
        # Thus, sorting by the start also sorts by the end, and we can
        # binary search on both to get the bounds of the overlapping segments.
        # Overlapping segments are guaranteed to be a contiguous set of rows
        # in the segment array for this reason.
        start = searchsorted(segments['end'], supercontig.start, side="right")
        end = searchsorted(segments['start'], supercontig.end, side="left")
        if end > start:
            yield supercontig, segments[start:end]

## Ensures output directory exists and has appropriate permissions
def setup_directory(dirname):
    if not os.path.isdir(dirname):
        try:
            os.mkdir(dirname)
        except IOError:
            die("Error: Could not create output directory: %s" % (dirname))
    else:
        # Require write and execute permissions on existing dir
        if not os.access(dirname, os.W_OK | os.X_OK):
            die("Error: Output directory is not writeable and executable.")

## Given labels and mnemonics, returns an ordered list of label_keys
##   and a new labels dict mapping label_keys to label strings
## If no mnemonics are specified, returns the passed labels and
##   a label_key ordering
def get_ordered_labels(labels, mnemonics=[]):
    if mnemonics is not None and len(mnemonics) > 0:
        # Create key lookup dictionary
        key_lookup = {}  # old_label -> label_key
        for label_key, label in labels.iteritems():
            assert(label not in key_lookup)  # Enforce 1-to-1
            key_lookup[label] = label_key

        labels = {}
        ordered_keys = []
        for old_label, new_label in mnemonics:
            label_key = key_lookup[old_label]
            ordered_keys.append(label_key)
            labels[label_key] = new_label
    else:
        # Don't change labels, but specify ordering
        partial_int_labels = {}
        for key, label in labels.iteritems():
            try:
                partial_int_labels[int(label)] = key
            except ValueError:
                partial_int_labels[label] = key
        ordered_keys = list(partial_int_labels[key]
                            for key in sorted(partial_int_labels.keys()))

    return ordered_keys, labels

## Maps the provided labels to mnemonics (or some other mnemonic file
##   field specified ready to be fed into R.
## Returns a numpy.array of strings with a row of [oldlabel, newlabel] for
## every old_label, and their ordering specifies the desired display order
## Labels should be a dict of label_keys -> label strings
def map_mnemonics(labels, mnemonicfilename, field="new"):
    if not mnemonicfilename:
        return []

    label_order, label_mnemonics = load_mnemonics(mnemonicfilename)
    str_labels = labels.values()
    mnemonics = []
    # Add mapping for labels in mnemonic file
    for old_label in label_order:
        try:
            new_label = label_mnemonics[old_label][field]
        except KeyError:
            die("Mnemonic file missing expected column: %s" % field)

        if old_label in str_labels:
            mnemonics.append([old_label, new_label])

    # Add mapping for labels not in mnemonic file
    # Use ordering of label mapping
    for label_key in sorted(labels.keys()):
        label = labels[label_key]  # Get actual label (string)
        if label not in label_order:  # Then map to same name
            mnemonics.append([label, label])

    return array(mnemonics)

def make_comment_ignoring_dictreader(iterable, *args, **kwargs):
    return DictReader((item for item in iterable if not item.startswith("#")),
                      *args, **kwargs)

## Loads segmentation label descriptions and mnemonics
##   from a tab-delimited file with a header row
def load_mnemonics(filename):
    """
    Input file should have a tab-delimited row for each label, of the form:
               old    new    description
      e.g.     4    IS   initiation (strong)
    Returns a tuple of (label_order, label_mnemonics)

    label_order: an ordered list of old labels,
      corresponding to the preferred display order in plots

    label_mnemonics: dict
      key: a string corresponding to an "old" label
      value: a dict with fields including "new" and "description",
        where description is a several-word description of the label
        and new is a few-character identifier
    """
    if filename is None:
        return []
    elif not os.path.isfile(filename):
        die("Could not find mnemonic file: %s" % filename)

    label_order = []
    label_mnemonics = {}
    with open(filename, "rU") as ifp:
        reader = make_comment_ignoring_dictreader(ifp)
        for row in reader:
            try:
                label_index = row["old"]
            except KeyError:
                die("Mnemonic file missing required column: 'old'")

            label_order.append(label_index)
            label_mnemonics[label_index] = row

    return (label_order, label_mnemonics)



#!/usr/bin/env python
from __future__ import with_statement

import inspect
import os
import sys
import unittest

from tempfile import NamedTemporaryFile

from segtools.feature_distance import DELIM, feature_distance, STANDARD_FIELDS

def data2bed(lines):
    # Input lines should each be a tuple of (chrom, start, end, label)
    strings = []
    for line in lines:
        strings.append("chr%s\t%d\t%d\t%s" % line)
    return "\n".join(strings)

def data2gff(lines):
    # Input lines should each be a tuple of (chrom, start, end, label[, strand])
    strings = []
    for line in lines:
        source = "source"
        chr, start, end, label = line[0:4]
        start += 1
        chr = "chr%s" % chr

        fields = [chr, source, label, start, end]
        # Optionally add score and strand
        try:
            strand = line[4]
            fields.extend([".", strand])
        except IndexError:
            pass

        field_strs = [str(field) for field in fields]
        strings.append("\t".join(field_strs))

    return "\n".join(strings)

class OutputSaver(object):
    """Class to validate the stout behavior of a call

    Suggested usage:
      saver = OutputSaver()  # Start recording stdout
      <commands with stout recorded>
      saver.restore()  # Stop recording stdout
      log = saver.log()  # Retrieve stdout recording
    """
    def __init__(self):
        self._stdout = sys.stdout
        sys.stdout = self
        self._log = []
    def write(self, string):
        self._log.append(string)
    def writelines(self, lines):
        self._log.extend(lines)
    def restore(self):
        sys.stdout = self._stdout
    def log(self):
        return "".join(self._log)

class MainTester(unittest.TestCase):
    def init(self):
        pass

    def setUp(self):
        self.kwargs = {"verbose": False}
        self.init()

        # Set self.features from self.segments, self.features_list
        self.feature_files = []
        self.segment_file = None
        self._open_files = []

        new_file = NamedTemporaryFile(suffix=".bed")
        new_file.write(data2bed(self.segments))
        new_file.flush()
        self._open_files.append(new_file)
        self.segment_file = new_file.name

        for type, data in self.features_list:
            if type == "bed":
                new_file = NamedTemporaryFile(suffix=".bed")
                new_file.write(data2bed(data))
            elif type == "gff":
                new_file = NamedTemporaryFile(suffix=".gff")
                new_file.write(data2gff(data))
            elif type == "gtf":
                new_file = NamedTemporaryFile(suffix=".gtf")
                new_file.write(data2gff(data))

            new_file.flush()
            self._open_files.append(new_file)
            self.feature_files.append(new_file.name)

    def tearDown(self):
        for file in self._open_files:
            file.close()

    def test(self):
        # Run function, logging stdout
        self._saver = OutputSaver()
        stats = feature_distance(self.segment_file, self.feature_files,
                                 **self.kwargs)
        self._saver.restore()
        log_lines = self._saver.log().strip("\n").split("\n")
        log_header = log_lines.pop(0).split(DELIM)

        # Check header
        header_answer = STANDARD_FIELDS + \
            [os.path.basename(filename) for filename in self.feature_files]
        self.assertEqual(log_header, header_answer)

        # Check values
        observed_strs = [line.split(DELIM)[len(STANDARD_FIELDS):]
                         for line in log_lines]
        answer_strs = [[str(val) for val in line] for line in self.answer]

        self.assertEqual(observed_strs, answer_strs)

class TestSimple(MainTester):
    def init(self):
        self.segments = [(1, 5, 6, 1),
                         (1, 10, 15, 1),
                         (1, 15, 20, 1)]
        self.features_list = [("bed", [(1, 6, 11, 1),
                                       (1, 11, 12, 1)])]
        self.answer = [[1], [0], [4]]

class TestStrandCorrect(MainTester):
    def init(self):
        self.kwargs["correct_strands"] = [0]
        self.segments = [(1, 60, 90, 0),
                         (1, 130, 140, 0)]
        self.features_list = [("gtf", [(1, 1, 10, 1, "+"),
                                       (1, 2, 3, 2, "-"),
                                       (1, 3, 50, 4, "+"),
                                       (1, 120, 125, 1, "-")])]
        self.answer = [[-11], [6]]

def suite():
    classes = []
    members = inspect.getmembers(sys.modules[__name__])
    for name, value in members:
        if inspect.isclass(value) and name.startswith("Test"):
            classes.append(value)

    tests = map(unittest.TestLoader().loadTestsFromTestCase, classes)
    return unittest.TestSuite(tests)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())



#!/usr/bin/env python
"""
If Segtools is taking too long to parse your large segmentation or
annotation file (specify --annotation), this
tool allows you to pre-process that file only once
and have it load much faster in the future.
INFILE will be parsed to create a special binary file with
a name of the form: "INFILE.pkl" or "OUTFILE.pkl".
Then, you can substitute this
new file for the corresponding SEGMENTATION or ANNOTATION
argument in future Segtools calls and Segtools will parse
it in much faster (but be sure to keep the ".pkl" extension intact)!
"""

from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

import sys

from . import Annotation, Segmentation, add_common_options

def preprocess(infilename, outfilename=None, annotation=False, verbose=False, clobber=False):
    if annotation:
        file_type = Annotation
    else:
        file_type = Segmentation

    print >>sys.stderr, ("Making %s object."
                         " See --help for more options"
                         % file_type.__name__)
    parsed_obj = file_type(infilename, verbose=verbose)
    parsed_obj.pickle(outfilename, verbose=verbose, clobber=clobber)

def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTIONS] INFILE [OUTFILE]"
    description = __doc__.strip()
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version,
                          description=description)

    add_common_options(parser, ['clobber', 'quiet'])
    parser.add_option("--annotation", action="store_true",
                      dest="annotation", default=False,
                      help="Read INFILE as an annotation, rather"
                      " than as a segmentation (default).")

    (options, args) = parser.parse_args(args)

    if len(args) < 1 or len(args) > 2:
        parser.error("Inappropriate number of arguments")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    kwargs = dict(options.__dict__)
    preprocess(*args, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import with_statement

import inspect
import sys
import unittest

from tempfile import NamedTemporaryFile

from segtools import Segmentation
from segtools.bed_compare import edit_distance

def beddata2bed(lines):
    # Input lines should each be a tuple of (chrom, start, end, label)
    strings = []
    for line in lines:
        strings.append("chr%s\t%d\t%d\t%s" % line)
    return "\n".join(strings)

class BedGenerator(unittest.TestCase):
    def init(self):
        pass

    def setUp(self):
        self.init()

        # Create fake bed files
        self._open_bedfiles = [NamedTemporaryFile(suffix=".bed"),
                               NamedTemporaryFile(suffix=".bed")]
        for file, data in zip(self._open_bedfiles, self.beddata):
            file.write(beddata2bed(data))
        for file in self._open_bedfiles:
            file.flush()
        self.bedfiles = [file.name for file in self._open_bedfiles]

    def tearDown(self):
        for file in self._open_bedfiles:
            file.close()

class EditDistanceTester(BedGenerator):
    def test(self):
        stats = edit_distance(self.bedfiles[0], self.bedfiles[1], verbose=False)
        self.assertValuesEqual(stats, self.answer)

    def assertValuesEqual(self, observed, expected):
        for val1, val2 in zip(observed, expected):
            self.assertEqual(val1, val2)

class TestPerfectOverlap(EditDistanceTester):
    def init(self):
        self.beddata = [[(1, 0, 50, 1),
                         (1, 50, 100, 2)],
                        [(1, 0, 50, 1),
                         (1, 50, 100, 2)]]
        self.answer = (100, 0, 0, 0)

class TestNoOverlap(EditDistanceTester):
    def init(self):
        self.beddata = [[(1, 10, 20, 1)],
                        [(1, 20, 35, 6)]]
        self.answer = (0, 0, 15, 10)

class TestSimpleDiff(EditDistanceTester):
    def init(self):
        self.beddata = [[(1, 11, 20, 1),
                         (1, 25, 35, 2)],
                        [(1, 15, 37, 2)]]
        self.answer = (10, 5, 7, 4)

class TestNotSegmentation(EditDistanceTester):
    def init(self):
        self.beddata = [[(1, 0, 10, 1),
                         (1, 5, 10, 1)],
                        [(1, 20, 35, 6)]]

    def test(self):
        self.assertRaises(Segmentation.SegmentOverlapError, edit_distance,
                          self.bedfiles[0], self.bedfiles[1], verbose=False)

def suite():
    classes = []
    members = inspect.getmembers(sys.modules[__name__])
    for name, value in members:
        if inspect.isclass(value) and name.startswith("Test"):
            classes.append(value)

    tests = map(unittest.TestLoader().loadTestsFromTestCase, classes)
    return unittest.TestSuite(tests)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())



#!/usr/bin/env python
from __future__ import division

__version__ = "$Revision: 228 $"

# Copyright 2008 Michael M. Hoffman <mmh1@washington.edu>

import sys

FIELDNAMES = ["seqname", "source", "feature", "start", "end",  # required
              "score", "strand", "frame", "group"]

class Datum(object):
    def __init__(self, words, *args, **kwargs):
        self.__dict__ = dict(zip(FIELDNAMES, words))
        self._words = tuple(words)

    def __repr__(self):
        return "%s%s" % (self.__class__.__name__, self._words)

class NativeDatum(Datum):
    def __init__(self, *args, **kwargs):
        Datum.__init__(self, *args, **kwargs)

        # convert to zero-based, half-open:
        # http://genome.ucsc.edu/FAQ/FAQformat#format3
        try:
            self.start = int(self.start) - 1
            self.end = int(self.end)
            try:
                self.score = float(self.score)
            except AttributeError:
                pass
            except ValueError:
                pass

            # attributes: http://genome.ucsc.edu/FAQ/FAQformat#format4
            if "gtf" in kwargs and kwargs["gtf"]:
                self.attributes = {}
                for attribute in self.group.rstrip("; ").split("; "):
                    type, value = attribute.split(" ", 1)
                    self.attributes[type] = value.strip('"')  # Remove quotes

                assert "gene_id" in self.attributes and \
                    "transcript_id" in self.attributes

            self._words = ((self.seqname, self.start, self.end)
                           + self._words[3:])
        except:
            print >>sys.stderr, "Error processing line: %s" % \
                  "\t".join(self._words)
            raise

def read(iterator, datum_cls=Datum, *args, **kwargs):
    try:
        for line in iterator:
            line = line.split("#", 1)[0].rstrip() # Ignore comments
            if not line: continue

            words = line.split("\t")  # Tab-delimited
            if words[0] == "track":  # Ignore any track lines
                continue

            assert len(words) >= 5

            yield datum_cls(words, *args, **kwargs)
    except:
        print >>sys.stderr, "Error processing line: %s" % line
        raise

def read_native(*args, **kwargs):
    return read(datum_cls=NativeDatum, *args, **kwargs)

def main(args=sys.argv[1:]):
    pass

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

import os
import re
import sys
import unittest

from pkg_resources import resource_listdir

def main(verbose=True):
    # Move to test directory to allow imports
    os.chdir(os.path.dirname(__file__))

    # Gather a list of unittest modules
    filenames = os.listdir(os.getcwd())
    regex = re.compile("^test_.*\.py$", re.IGNORECASE)
    module_filenames = filter(regex.search, filenames)
    make_module_name = lambda filename: filename[:-3]
    modulenames = map(make_module_name, module_filenames)
    if verbose:
        print "Found test modules: %r" % modulenames

    # Run the test suite for each
    suite = unittest.defaultTestLoader.loadTestsFromNames(modulenames)
    if verbose:
        verbosity = 2
    else:
        verbosity = 1

    unittest.TextTestRunner(verbosity=verbosity).run(suite)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
This module aggregates segmentation data around features, generating
a histogram for each segmentation label that shows the frequency of
observing that label at that position relative the the feature.

If using gene mode, the input file should have features with names:
exon, start_codon, CDS
as provided by exporting UCSC gene data in GTF format.
"""

# A package-unique, descriptive module name used for filenames, etc
MODULE="feature_aggregation"

__version__ = "$Revision: 339 $"

import os
import sys

from collections import defaultdict
from functools import partial
from numpy import arange, array, empty, linspace, round

from . import (Annotation, log, Segmentation, DTYPE_SEGMENT_START,
               DTYPE_SEGMENT_END, DTYPE_SEGMENT_KEY, DTYPE_STRAND,
               die, RInterface, open_transcript, add_common_options)
from .common import (fill_array, get_ordered_labels, make_tabfilename,
                     map_segment_label, setup_directory, tab_saver)
from .html import save_html_div

NAMEBASE = "%s" % MODULE
NAMEBASE_SPLICING=os.extsep.join([NAMEBASE, "splicing"])
NAMEBASE_TRANSLATION=os.extsep.join([NAMEBASE, "translation"])

STATIC_FIELDNAMES = ["group", "component", "offset"]

HTML_TITLE_BASE = "Feature aggregation"

# REGION_MODE, GENE_MODE mean do quantile bins
POINT_MODE = "point"
REGION_MODE = "region"
GENE_MODE = "gene"
MODES = [POINT_MODE, REGION_MODE, GENE_MODE]
DEFAULT_MODE = POINT_MODE

# Component names
#   A single %f/%d/%s will automatically be substituted with the average number
#   of bases in that component over all features
FLANK_5P = "5' flanking: %d bp"
FLANK_3P = "3' flanking: %d bp"
FLANK_COMPONENTS = [FLANK_5P, FLANK_3P]
POINT_COMPONENTS = FLANK_COMPONENTS

REGION_COMPONENT = "internal (%d bp)"
REGION_COMPONENTS = [FLANK_5P, REGION_COMPONENT, FLANK_3P]

INITIAL_EXON = "initial exon (%d bp)"
TERMINAL_EXON = "terminal exon (%d bp)"
SPLICE_COMPONENTS = [
    INITIAL_EXON,
    "initial intron (%d bp)",
    "internal exons (%d bp)",
    "internal introns (%d bp)",
    TERMINAL_EXON,
    "terminal intron (%d bp)"]
CODING_COMPONENTS = [
    "initial 5' UTR (%d bp)",
    "5' UTR introns (%d bp)",
    "internal 5' UTR (%d bp)",
    "terminal 5' UTR (%d bp)",
    "initial CDS (%d bp)",
    "terminal CDS (%d bp)",
    "initial 3' UTR (%d bp)",
    "internal 3' UTR (%d bp)",
    "3' UTR introns (%d bp)",
    "terminal 3' UTR (%d bp)"]

EXON_COMPONENTS = [FLANK_5P] + SPLICE_COMPONENTS + [FLANK_3P]
GENE_COMPONENTS = EXON_COMPONENTS + CODING_COMPONENTS

# For parsing GTF file
EXON_FEATURE = "exon"
CDS_FEATURE = "CDS"
MIN_EXONS = 1
MIN_CDSS = 0

# Default parameter values
FLANK_BASES = 500
REGION_BINS = 50
INTRON_BINS = 50
EXON_BINS = 25

R = RInterface(["common.R", "aggregation.R"])

class GeneAnnotation(Annotation):
    """Annotation class for gene-type annotation"""

    def _from_file(self, filename, verbose=True):
        """Load the gtf file in terms of idealized gene features.

        Parses the given feature file and replaces feature names from:
          CDS
          exon
          (other features ignored)
        with feature components from:
          GENE_COMPONENTS

        Field 8 of the feature file must be a list of semi-colon delimited
        properties and gene_id must be the first one of them.

        All features must be stranded.

        """
        from .common import inverse_dict

        log("  Parsing GTF file...", verbose, end="")
        format = self._get_file_format(filename)
        if format != "gtf":
            raise self.FormatError("ANNOTATION must be in GTF format to"
                                   " aggregate with --mode=gene")

        # Start with establishing a dict:
        #   gene_id -> dict(transcript_id -> (chrom, start, end, name, strand))
        #   start is 0-indexed, end is non-inclusive (BED)
        gene_dict = defaultdict(partial(defaultdict, list))
        for row in self._iter_rows(filename, verbose=verbose):
            if row['strand'] not in set(["+", "-"]):
                raise self.FormatError("ANNOTATION must specify a strand"
                                       " for every entry to aggregate with"
                                       " --mode=gene")

            # Add feature to dict
            gene_dict[row['gene_id']][row['transcript_id']].append(row)

        log(" done", verbose)
        log("  Mapping onto gene model...", verbose, end="")

        # Eventually create feature dict: chrom -> list(features)
        data = defaultdict(list)
        label_dict = {}
        n_genes = 0
        n_label_segments = {}
        n_label_bases = {}
        for gene_id, transcript_dict in gene_dict.iteritems():
            # Select only longest transcript
            transcript_id, transcript = \
                self._get_longest_transcript(transcript_dict)
            gene_model = self._transcript_to_gene_features(transcript)

            if gene_model is not None:
                n_genes += 1
                # Add features to a normal, chrom-based feature dict
                for gene_part in gene_model:
                    # label is the component of that gene part
                    chrom, start, end, strand, label = gene_part
                    try:
                        label_key = label_dict[label]
                    except KeyError:
                        label_key = len(label_dict)
                        label_dict[label] = label_key
                        n_label_segments[label] = 0
                        n_label_bases[label] = 0

                    feature = (start, end, label_key, strand)
                    data[chrom].append(feature)
                    n_label_segments[label] += 1
                    n_label_bases[label] += end - start
#            raw_input()

        labels = inverse_dict(label_dict)
        dtype = [('start', DTYPE_SEGMENT_START),
                 ('end', DTYPE_SEGMENT_END),
                 ('key', DTYPE_SEGMENT_KEY),
                 ('strand', DTYPE_STRAND)]

        chromosomes = dict((chrom, array(features, dtype=dtype))
                           for chrom, features in data.iteritems())

        log(" done", verbose)
        log("  Sorting...", verbose, end="")

        # Sort features by ascending start
        for chrom_features in chromosomes.itervalues():
            chrom_features.sort(order=['start'])

        log(" done", verbose)

        self.filename = filename
        self.chromosomes = chromosomes
        self._labels = labels
        self._n_label_segments = n_label_segments
        self._n_label_bases = n_label_bases
        self._n_genes = n_genes
        self._inv_labels = label_dict

    def num_genes(self):
        return self._n_genes

    def _preprocess_entries(self, rows):
        """Convert list of gene feature entries to processed gene data.

        If multiple transcripts are found, only the longest is taken.

        :param entries: each entry is a dict
                        and all entries should correspond to the same gene_id
        :returns: chrom, strand, list(exon), list(cds)

        """
        # Remove list of gene features and preprocess
        #   (extracting constant strand, chrom)
        # Returns gene info and list of exons and CDSs
        gene_strand = None
        gene_chrom = None
        exons = []
        cdss = []

        for row in rows:
            chrom = row['chrom']
            start = row['start']
            end = row['end']
            strand = row['strand']
            name = row['name']

            # Ensure strand and chrom match rest for this gene
            if gene_strand is None:
                gene_strand = strand
            elif gene_strand != strand:
                die("Found gene features on more than one strand: [%s, %s]\n%s"
                    % (gene_strand, strand, row))

            if gene_chrom is None:
                gene_chrom = chrom
            elif gene_chrom != chrom:
                die("Found gene features on more than one chromosome:"
                    " [%s, %s]\n%s" % (gene_chrom, chrom, row))

            partial_entry = (start, end)
            if name == EXON_FEATURE:
                exons.append(partial_entry)
            elif name == CDS_FEATURE:
                cdss.append(partial_entry)
        return gene_chrom, gene_strand, exons, cdss

    def _transcript_to_gene_features(self, rows):
        """Returns a set of gene-model entries based upon the given entries.

        Interprets the given set of entries in terms of an idealized gene model
        with 5' and 3' UTRs, initial, internal, and terminal exons and introns.

        :param entries: see preprocess_entries for a description
        :returns: a new set of entries based upon this gene model
                  or None if the entries don't fit the model.
        """

        chrom, strand, exons, cdss = self._preprocess_entries(rows)

        # Ignore genes without any exons
        if not exons:
            return None

        exons.sort()
        cdss.sort()

        # Create introns between every pair of now-sorted exons
        introns = []
        prev_end = None
        for exon_start, exon_end in exons:
            if prev_end is not None:
                introns.append((prev_end, exon_start))
            prev_end = exon_end

        # Flip directions for '-' strand
        if strand == "-":
            exons.reverse()
            introns.reverse()
            cdss.reverse()

        features = []
        # Add gene features to new list, renaming components
        #   according to an idealized gene model
        # Assumes exons and cdss in sorted order (ensured by sort() above)

        # Binds local variables: chrom, strand
        def add_feature(component, start, end):
            """Makes tuple feature for feature list"""
            features.append((chrom, start, end, strand, component))  # tuple

        # first exon
        add_feature(SPLICE_COMPONENTS[0], *exons[0])
        # first intron
        if len(introns) > 0:
            add_feature(SPLICE_COMPONENTS[1], *introns[0])
        # internal exons
        for exon in exons[1:-1]:
            add_feature(SPLICE_COMPONENTS[2], *exon)
        # internal introns
        for intron in introns[1:-1]:
            add_feature(SPLICE_COMPONENTS[3], *intron)
        # last intron
        if len(introns) > 0:
            add_feature(SPLICE_COMPONENTS[5], *introns[-1])
        # last exon
        add_feature(SPLICE_COMPONENTS[4], *exons[-1])


        # If there were no CDS's, then there is no point in dealing with UTR/CDS
        #   regions, so skip all that.
        if len(cdss) == 0:
            return features

        # Binds local variable strand to strand-correct
        def upstream(x, y):
            """Returns true if x starts before (is 5') of y, false otherwise"""
            return (strand == "+" and x[0] < y[0]) or \
                (strand == "-" and x[1] > y[1])

        # Go through exons and introns and pick out those that are in UTR
        # Exons are a little trickier because the UTR region has to be trimmed
        #   to the CDS if they overlap
        UTR5p_exons = []
        UTR5p_introns = []
        UTR3p_exons = []
        UTR3p_introns = []
        first_cds = cdss[0]
        last_cds = cdss[-1]
        first_cds_start, first_cds_end = first_cds
        last_cds_start, last_cds_end, = last_cds
        for exon in exons:
            exon_start, exon_end = exon
            # See if exon is in UTR relative to CDSs, and trim to UTR fragment
            utr_list = None
            if strand == "+":
                if exon_start < first_cds_start:
                    exon_end = min(exon_end, first_cds_start)
                    utr_list = UTR5p_exons
                elif exon_end > last_cds_end:
                    exon_start = max(exon_start, last_cds_end)
                    utr_list = UTR3p_exons
            else:  # strand == "-":
                if exon_end > first_cds_end:
                    exon_start = max(exon_start, first_cds_end)
                    utr_list = UTR5p_exons
                elif exon_start < last_cds_start:
                    exon_end = min(exon_end, last_cds_start)
                    utr_list = UTR3p_exons
            # Add if exon was in UTR and has positive length
            if utr_list is not None and exon_start < exon_end:
                utr_list.append((exon_start, exon_end))  # Append tuple

        for intron in introns:
            if upstream(intron, first_cds):
                UTR5p_introns.append(intron)
            elif upstream(last_cds, intron):
                UTR3p_introns.append(intron)


        # Go through the list of UTR elements and add them to the feature list
        #   with appropriate component names
        # initial 5' UTR
        if len(UTR5p_exons) > 0:
            add_feature(CODING_COMPONENTS[0], *UTR5p_exons[0])
        # 5' UTR introns
        for intron in UTR5p_introns:
            add_feature(CODING_COMPONENTS[1], *intron)
        # internal 5' UTR
        for exon in UTR5p_exons[1:-1]:
            add_feature(CODING_COMPONENTS[2], *exon)
        # terminal 5' UTR
        if len(UTR5p_exons) > 0:
            add_feature(CODING_COMPONENTS[3], *UTR5p_exons[-1])
        # first CDS
        add_feature(CODING_COMPONENTS[4], *first_cds)
        # last CDS
        add_feature(CODING_COMPONENTS[5], *last_cds)
        # initial 3' UTR
        if len(UTR3p_exons) > 0:
            add_feature(CODING_COMPONENTS[6], *UTR3p_exons[0])
        # internal 3' UTR
        for exon in UTR3p_exons[1:-1]:
            add_feature(CODING_COMPONENTS[7], *exon)
        # 3' UTR introns
        for intron in UTR3p_introns:
            add_feature(CODING_COMPONENTS[8], *intron)
        # terminal 3' UTR
        if len(UTR3p_exons) > 0:
            add_feature(CODING_COMPONENTS[9], *UTR3p_exons[-1])

#         for row in rows:
#             log("%s\t%d\t%d\t%s" % (row['chrom'], row['start'],
#                                     row['end'], row['name']))
#         for feature in features:
#             log("\t%s" % str(feature))
#         raw_input()
        return features

    @staticmethod
    def _get_longest_transcript(transcript_dict):
        """Return the longest transcript in the dict.

        :type transcript_dict: transcript_id -> entries
        :returns: tuple of (transcript_id, entries) for the longest transcript
                  or None if there were no transcripts.
        """

        def get_transcript_length(rows):
            start = min(row['start'] for row in rows)
            end = max(row['end'] for row in rows)

            return end - start

        # XXX: this could probably be simplified vastly using max()
        # and a generator expression; the semantics would change
        # slightly in case of ties; you would get the transcript with
        # the highest ID instead of the first one of that length.
        # check to see if this is OK
        max_length = 0
        res = None
        for id, rows in transcript_dict.iteritems():
            length = get_transcript_length(rows)
            if length > max_length:
                max_length = length
                res = (id, rows)
        return res


def print_bed_from_gene_component(features, component="terminal exon"):
    with open("%s.bed" % component, "w") as ofp:
        for chrom, chrom_features in features.iteritems():
            for feature in chrom_features:
                if feature["name"].startswith(component):
                    print >>ofp, "%s\t%s\t%s\t%s" % (chrom,
                                                     feature["start"],
                                                     feature["end"],
                                                     feature["name"])

def calc_bin_starts(start, end, num_bins):
    """Calculate internal bin locations"""
    if num_bins <= 0:
        return array([], dtype="int")

    # linspace: Return evenly spaced numbers over a specified interval.
    # endpoint=False
    bin_starts_unrounded = linspace(start, end, num_bins, False)

    res = empty(bin_starts_unrounded.shape, dtype="int")
    round(bin_starts_unrounded, out=res)
    return res

def calc_feature_windows(feature, labels, mode, component_bins):
    """Calculate where the bin windows for this feature begin; Spread
    internal bins throughout feature

    Return a list of tuples: (component, window)

    component: str (usually from *_COMPONENT/*_COMPONENTS constants)
    window: numpy.ndarray(dtype=int) of base indices
    """
    name = labels[feature['key']]
    start = feature['start']
    end = feature['end']
    try:
        strand = feature['strand']
    except (IndexError, KeyError):
        assert mode != GENE_MODE
        strand = "."

    length = end - start
    assert length >= 0

    # Include flanks by default
    num_5p_bins = component_bins[FLANK_5P]
    num_3p_bins = component_bins[FLANK_3P]

    # remove flanks if gene mode and not on one end or other
    if mode == GENE_MODE:
        assert name in component_bins
        component = name
        if component != INITIAL_EXON:
            # Not initial exon, so no 5' flank
            num_5p_bins = 0

        if component != TERMINAL_EXON:
            # Not terminal exon, so no 3' flank
            num_3p_bins = 0
    elif mode == REGION_MODE:
        component = REGION_COMPONENT
    else:
        assert mode == POINT_MODE
        component = None
        num_internal_bins = 0

    if component is not None:
        # default value of 0
        num_internal_bins = component_bins.get(component, 0)

    if num_internal_bins > length:
        # don't calculate internal bins in this case
        #print >> sys.stderr, "Warning: %d %s bins > %d bases" % \
        #    (num_internal_bins, component, length)
        num_internal_bins = 0

    internal_bins = calc_bin_starts(start, end, num_internal_bins)

    # Calculate flanking base locations
    # XXX: what happens when the flanking regions are off the chromosome?
    if strand == "-":
        bins_5p = arange(end, end + num_5p_bins, dtype="int")[::-1]
        internal_bins = internal_bins[::-1]  # reverse
        bins_3p = arange(start - num_3p_bins, start, dtype="int")[::-1]
    else:
        bins_5p = arange(start - num_5p_bins, start, dtype="int")
        bins_3p = arange(end, end + num_3p_bins, dtype="int")

    res = []
    if len(bins_5p) > 0:
        res.append((FLANK_5P, bins_5p))
    if len(internal_bins) > 0:
        res.append((component, internal_bins))
    if len(bins_3p) > 0:
        res.append((FLANK_3P, bins_3p))

    #for key, val in locals().iteritems():
    #    print >>sys.stderr, "%s : %s" % (key, val)
    #sys.exit(1)

    return res

## Accepts data from dict: chr -> dict {"start", "end", "strand"})
##   zero-based start and end (exclusive) indices
## If components is:
##   [] or None, aggregation will be just over the flanking regions
##   length 1, aggregation will be in "region" mode
##   length > 1, aggregation will be over each component separately, with
##     flanking regions before the 1st and after the last component in the list
##     Each feature's component entry must match one of these exactly
## component_bins is a dict: component -> number of bins for that component
## If not by_groups: all groups found are treated as only group in groups
## Returns:
##   groups: a list of the groups aggregated over
##   counts: dict(label_key -> dict(feature -> dict(component -> histogram)))
def calc_aggregation(segmentation, features, mode, groups, components,
                     component_bins, quick=False, by_groups=False,
                     verbose=True):
    if not by_groups:
        assert len(groups) == 1
    else:
        assert len(groups) > 0

    labels = segmentation.labels

    if verbose:
        log("\tGroups: %s" % groups)
        log("\tComponents and bins:")
        for component in components:
            log("\t\t", end="")
            log("%s; %d bins" % (component, component_bins[component]))

    # counts is an accumulator
    # dict:
    #   key: feature_group
    #   value: dict:
    #      key: component_name
    #      value: numpy.ndarray histogram [bin, label_key]
    counts = dict([(group,
                    dict([(component, fill_array(0, (bins, len(labels))))
                          for component, bins in component_bins.iteritems()]))
                   for group in groups])

    counted_features = 0

    for chrom, segments in segmentation.chromosomes.iteritems():
        log("\t%s" % chrom, verbose)

        try:
            chrom_features = features.chromosomes[chrom]
        except KeyError:
            continue
        else:
            if len(chrom_features) == 0:
                continue

        # Get bounds on segmentation for chr (since segments are sorted)
        segmentation_start = segments['start'][0]
        segmentation_end = segments['end'][-1]

        # Map entire chromosome's segments into an array of label_keys
        # XXXopt: this line takes up to 3.2 GB of memory
        segment_map, sentinel = map_segment_label(
            segments, (segmentation_start, segmentation_end))

        # For each feature, tally segments in window
        for feature in chrom_features:
            # XXXopt: Process every feature, since we don't know how wide the
            # aggregation window is.
            feature_counted = False
            if by_groups and mode != GENE_MODE:
                group = features.labels[feature['key']]
            else:
                group = groups[0]

            group_counts = counts[group]
            # dict:
            #      key: component_name
            #      value: numpy.ndarray histogram [bin, label_key]

            # Spread internal bins throughout feature if not POINT_MODE
            component_windows = calc_feature_windows(feature, features.labels,
                                                     mode, component_bins)

            # Scan window, tallying segments observed
            for component, window in component_windows:
                component_counts = group_counts[component]
                #log("component: %r" % component)
                #log("component_counts.shape: %r" % str(component_counts.shape))

                map_indices = window - segmentation_start
                count_indices = arange(0, len(window))

                # Lookup all label_keys within segmentation at once
                keep = ((window >= segmentation_start) &
                        (window < segmentation_end))
                label_keys = segment_map[map_indices[keep]]
                if len(label_keys) == 0:
                    continue
                count_indices = count_indices[keep]

                # Find indices where label_keys are set
                keep = (label_keys != sentinel).nonzero()[0]
                label_keys = label_keys[keep]
                if len(label_keys) == 0:
                    continue
                count_indices = count_indices[keep]

                #raw_input()
                component_counts[count_indices, label_keys] += 1
                if len(keep) > 0 and not feature_counted:
                    counted_features += 1
                    feature_counted = True

        if quick:
            break

    return (counts, counted_features)

def make_row(labels, row_data):
    values = {}
    for label_key, label in labels.iteritems():
        values[label] = row_data[label_key]

    return values

## Saves the data to a tab file
def save_tab(segmentation, features, counts, components, component_bins,
             counted_features, dirpath, mode, verbose=True,
             clobber=False, namebase=NAMEBASE):
    metadata = {"num_features": counted_features}
    # Add metadata to tell R how to display gene model.
    if mode == GENE_MODE:
        metadata["spacers"] = len(EXON_COMPONENTS)

    label_keys, labels = get_ordered_labels(segmentation.labels)
    for label_key in label_keys:
        label = labels[label_key]
        assert label not in STATIC_FIELDNAMES
        assert label not in metadata
        metadata[label] = segmentation.num_label_bases(label)

    log("Saving metadata: %r" % metadata, verbose)

    fieldnames = STATIC_FIELDNAMES + [labels[label_key]
                                      for label_key in label_keys]
    with tab_saver(dirpath, namebase, fieldnames=fieldnames, metadata=metadata,
                   clobber=clobber, verbose=verbose) as saver:
        # Compute the average length for features in each component,
        #   merging across groups.
        avg_component_lengths = {}
        for component in components:
            avg_len = None
            if component in FLANK_COMPONENTS:
                avg_len = component_bins[component]
            else:
                sum = 0
                count = 0
                for chrom_features in features.chromosomes.itervalues():
                    if component == REGION_COMPONENT:
                        # Merge groups
                        sum += (chrom_features['end'] -
                                chrom_features['start']).sum()
                        count += len(chrom_features)
                    elif component in GENE_COMPONENTS:
                        # Separate by component (key column)
                        try:
                            label_key = features.label_key(component)
                        except KeyError:
                            continue

                        keep = chrom_features['key'] == label_key
                        sum += (chrom_features['end'][keep] -
                                chrom_features['start'][keep]).sum()
                        count += keep.sum()

                if sum > 0:
                    avg_len = sum / count

            avg_component_lengths[component] = avg_len

        for group in counts:
            for component in components:
                hist = counts[group][component]
                # Try to substitute average lengths into component names
                component_name = component
                avg_len = avg_component_lengths[component]
                if avg_len is not None:
                    try:
                        component_name = component % avg_len
                    except TypeError:
                        pass

                if component == FLANK_5P:
                    offsets = range(-len(hist), 0)
                else:
                    offsets = range(0, len(hist))

                for offset, row_data in zip(offsets, hist):
                    row = make_row(labels, row_data)
                    row["group"] = group
                    row["component"] = component_name
                    row["offset"] = offset
                    saver.writerow(row)

## Plots aggregation data from tab file
def save_plot(dirpath, mode, namebase=NAMEBASE, clobber=False, verbose=True,
              mnemonic_file=None, normalize=False, significance=False,
              transcriptfile=None, ropts=None):
    R.start(verbose=verbose, transcriptfile=transcriptfile)

    tabfilename = make_tabfilename(dirpath, namebase)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    if not mnemonic_file:
        mnemonic_file = ""

    # Plot data in tab file
    if mode == GENE_MODE:
        R.plot("save.gene.aggregations", dirpath, NAMEBASE_SPLICING,
               NAMEBASE_TRANSLATION, tabfilename, mnemonic_file=mnemonic_file,
               normalize=normalize, clobber=clobber, significance=significance,
               ropts=ropts)
    else:
        R.plot("save.aggregation", dirpath, namebase, tabfilename,
               mnemonic_file=mnemonic_file, normalize=normalize,
               clobber=clobber, significance=significance, ropts=ropts)

def save_html(dirpath, featurefilename, mode, clobber=False,
              verbose=True, normalize=False):
    featurebasename = os.path.basename(featurefilename)
    title = "%s (%s)" % (HTML_TITLE_BASE, featurebasename)
    if normalize:
        yaxis = "enrichment"
    else:
        yaxis = "count"

    if mode == GENE_MODE:
        templatefile = "aggregation_gene_div.tmpl"
        extra_namebases = {"splice": NAMEBASE_SPLICING,
                           "trans": NAMEBASE_TRANSLATION}
    else:
        templatefile = "aggregation_div.tmpl"
        extra_namebases = {}

    save_html_div(templatefile, dirpath, NAMEBASE, clobber=clobber,
                  module=MODULE, featurefilename=featurebasename, mode=mode,
                  title=title, yaxis=yaxis, verbose=verbose,
                  extra_namebases=extra_namebases)

def print_array(arr, tag="", type="%d"):
    if len(arr) > 2:
        fstring = "%%s: \t%s, %s, ..., %s, ..., %s, %s" % tuple([type]*5)
        log(fstring % (tag,
                       arr[0],
                       arr[1],
                       arr[int(arr.shape[0] / 2)],
                       arr[-2],
                       arr[-1]))
    else:
        log(str(arr))

def get_components_and_bins(mode, flank_bases=FLANK_BASES,
                            region_bins=REGION_BINS,
                            intron_bins=INTRON_BINS,
                            exon_bins=EXON_BINS):
    """Get components and number of data points per component"""
    if mode == GENE_MODE:
        components = GENE_COMPONENTS
    elif mode == REGION_MODE:
        components = REGION_COMPONENTS
    else:
        assert mode == POINT_MODE
        components = POINT_COMPONENTS

    component_bins = {}
    for component in components:
        if component in FLANK_COMPONENTS:
            bins = flank_bases
        elif component in GENE_COMPONENTS:
            if "intron" in component:
                bins = intron_bins
            elif "exon" in component or \
                    "UTR" in component or \
                    "CDS" in component:
                # All UTR components that don't contain "intron" are exons
                bins = exon_bins
            else:
                bins = region_bins
        else:
            bins = region_bins

        assert component not in component_bins  # Keep unique
        component_bins[component] = bins

    return components, component_bins

## Package entry point
def validate(bedfilename, featurefilename, dirpath,
             flank_bases=FLANK_BASES, region_bins=REGION_BINS,
             intron_bins=INTRON_BINS, exon_bins=EXON_BINS,
             by_groups=False, mode=DEFAULT_MODE, clobber=False,
             quick=False, replot=False, noplot=False, normalize=False,
             significance=False, mnemonic_file=None, verbose=True,
             ropts=None):

    if not replot:
        setup_directory(dirpath)
        segmentation = Segmentation(bedfilename, verbose=verbose)

        if mode == GENE_MODE:
            features = GeneAnnotation(featurefilename, verbose=verbose)
            num_features = features.num_genes()
            groups = ["genes"]
        else:
            features = Annotation(featurefilename, verbose=verbose)
            num_features = features.num_segments()
            if by_groups:
                groups = features.labels.values()
            else:
                groups = ["features"]

        log("Aggregating over %d features" % num_features, verbose)

        components, component_bins = \
            get_components_and_bins(mode,
                                    flank_bases=flank_bases,
                                    region_bins=region_bins,
                                    intron_bins=intron_bins,
                                    exon_bins=exon_bins)

        res = calc_aggregation(segmentation, features, mode=mode,
                               groups=groups, components=components,
                               component_bins=component_bins, quick=quick,
                               by_groups=by_groups, verbose=verbose)
        counts, counted_features = res

        save_tab(segmentation, features, counts, components, component_bins,
                 counted_features, dirpath, mode, clobber=clobber,
                 verbose=verbose)

    if not noplot:
        with open_transcript(dirpath, MODULE) as transcriptfile:
            save_plot(dirpath, mode, clobber=clobber, verbose=verbose,
                      mnemonic_file=mnemonic_file, normalize=normalize,
                      significance=significance, transcriptfile=transcriptfile,
                      ropts=ropts)

    save_html(dirpath, featurefilename, mode=mode, verbose=verbose,
              clobber=clobber, normalize=normalize)

def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] SEGMENTATION ANNOTATION"
    description = ("Plot the enrichment of the SEGMENTATION labels relative"
                   " to the position of features in ANNOTATION."
                   " Features can be grouped by the"
                   " 'name'/'feature' column by supplying --groups.")
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version,
                          description=description)

    group = OptionGroup(parser, "Output options")
    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
    parser.add_option_group(group)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet', 'quick', 'replot',
                               'noplot'])

    group.add_option("--groups", action="store_true",
                     dest="by_groups", default=False,
                     help="Separate data into different groups based upon"
                     " ANNOTATION's 'name'/'feature' field"
                     " if --mode=region or --mode=point. Does"
                     " nothing if --mode=gene.")
    group.add_option("--normalize", action="store_true",
                     dest="normalize", default=False,
                     help="Plot the relative frequency of SEGMENTATION labels,"
                     " normalized by the number of segments in that group"
                     " instead of the raw counts"
                     " (normalize over SEGMENTATION labels)")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Main aggregation options")
    group.add_option("--mode", choices=MODES,
                     dest="mode", type="choice", default=DEFAULT_MODE,
                     help="one of: %s. [default: %%default]" % \
                     ", ".join(MODES))
    group.add_option("-f", "--flank-bases", metavar="N",
                     dest="flank_bases", type="int", default=FLANK_BASES,
                     help="Aggregate this many base pairs off each"
                     " end of feature or gene [default: %default]")
    group.add_option("-r", "--region-samples", type="int", metavar="N",
                     dest="region_bins", default=REGION_BINS,
                     help="If --mode=region, aggregate over each internal"
                     " feature by taking this many evenly-spaced samples."
                     " Warning: features with a length < N will be excluded"
                     " from the results [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Gene aggregation options")
    group.add_option("-i", "--intron-samples", type="int", metavar="N",
                     dest="intron_bins", default=INTRON_BINS,
                     help="Aggregate over each intron"
                     " by taking this many evenly-spaced samples"
                     " [default: %default]")
    group.add_option("-e", "--exon-samples", type="int", metavar="N",
                     dest="exon_bins", default=EXON_BINS,
                     help="Aggregate over each exon"
                     " by taking this many evenly-spaced samples"
                     " [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) != 2:
        parser.error("Inappropriate number of arguments")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    bedfilename = args[0]
    featurefilename = args[1]
    kwargs = dict(options.__dict__)
    outdir = kwargs.pop('outdir')

    validate(bedfilename, featurefilename, outdir, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

"""segtools: Tools for exploratory analysis of genomic segmentations

Segtools is a Python package designed to put genomic segmentations back
in the context of the genome! Using R for graphics, Segtools provides a
number of modules to analyze a segmentation in various ways and help you
interpret its biological relevance.
"""

__version__ = "1.1.8"

# Copyright 2008-2011 Michael M. Hoffman <mmh1@uw.edu>
# Copyright 2009-2011 Orion J. Buske <stasis@uw.edu>

import sys

# required for from __future__ import division, with_statement;
# relative imports
assert sys.version_info >= (2, 5, 1)

from ez_setup import use_setuptools
use_setuptools()

from setuptools import find_packages, setup

doclines = __doc__.splitlines()
name, short_description = doclines[0].split(": ")
long_description = "\n".join(doclines[2:])

url = "http://noble.gs.washington.edu/proj/%s/" % name.lower()
download_url = "%s%s-%s.tar.gz" % (url, name, __version__)

classifiers = ["Natural Language :: English",
               "Programming Language :: Python"]

entry_points = """
[console_scripts]
segtools-aggregation = segtools.aggregation:main
segtools-compare = segtools.compare:main
segtools-feature-distance = segtools.feature_distance:main
segtools-flatten = segtools.flatten:main
segtools-gmtk-parameters = segtools.gmtk_parameters:main
segtools-html-report = segtools.html:main
segtools-length-distribution = segtools.length_distribution:main
segtools-nucleotide-frequency = segtools.nucleotide_frequency:main [genomedata]
segtools-overlap = segtools.overlap:main
segtools-preprocess = segtools.preprocess:main
segtools-relabel = segtools.relabel:main
segtools-signal-distribution = segtools.signal_distribution:main [genomedata]
segtools-transition = segtools.transition:main
"""
#segtools = segtools.validate_all:main

install_requires = ["numpy>=1.3", "rpy2"]
# XXX: add optional requirement for PyGraphviz
extras_require = {'genomedata': "genomedata"}

if __name__ == "__main__":
    setup(name=name,
          version=__version__,
          description=short_description,
          author="Michael Hoffman",
          author_email="mmh1@uw.edu",
          maintainer="Orion Buske",
          maintainer_email="stasis@uw.edu",
          url=url,
          download_url=download_url,
          classifiers=classifiers,
          long_description=long_description,
          install_requires=install_requires,
          extras_require=extras_require,
          zip_safe=False,  # For R files to source others, they can't be zip'd
          packages=find_packages("."),
          include_package_data=True,
          entry_points=entry_points
          )



#!/usr/bin/env python

"""
Given a SEGMENTATION and ANNOTATION file, 1) prints the distance of each
segment to the nearest feature in the ANNOTATION file
(zero if the two overlap) and
2) generates a histogram of these distances.
Distance is the difference between the nearest bases of the segment and the
feature, so if there is one base pair between them, the distance is 2.
Distances can be strand-corrected with respect to stranded features
by specifying --stranded.
"""

from __future__ import division, with_statement

__version__ = "$Revision: 339 $"


import os
import sys
from time import time

from numpy import NaN, zeros

from . import Annotation, log, Segmentation, RInterface, open_transcript, \
     add_common_options
from .common import FeatureScanner, die, get_ordered_labels, \
    make_tabfilename, setup_directory, tab_saver
from .html import save_html_div

MODULE = "feature_distance"
NAMEBASE = MODULE
HTML_TITLE_BASE = "Distance to nearest feature"
HTML_TEMPLATE_FILENAME = "distance_div.tmpl"


PRINT_FIELDS = ["chrom", "start", "end", "name", "distance"]
TAB_FIELDS = ["label", "group", "distance", "count"]
DELIM = "\t"

DEFAULT_BINS = 100
DEFAULT_BIN_WIDTH = 100

R = RInterface(["common.R", "distance.R"])

def print_line(labels, chrom, segment, distance):
    fields = [chrom,
              str(segment['start']),
              str(segment['end']),
              labels[segment['key']],
              str(distance)]
    print DELIM.join(fields)

def save_tab(counts, bins, labels, groups, outdir,
             clobber=False, verbose=True):
    label_keys = sorted(labels.keys())
    group_keys = sorted(groups.keys())
    with tab_saver(outdir, NAMEBASE, TAB_FIELDS, verbose=verbose,
                   clobber=clobber) as saver:
        for label_key in label_keys:
            for group_key in group_keys:
                count_vector = counts[label_key, group_key, :]
                # Only print if there is at least one data point
                if count_vector.sum() == 0: continue
                for bin, count in zip(bins, count_vector):
                    row = dict(zip(TAB_FIELDS,
                                   [labels[label_key], groups[group_key],
                                    str(bin), str(count)]))
                    saver.writerow(row)

def save_plot(outdir, namebase=NAMEBASE, clobber=False, ropts=None,
              verbose=True, mnemonic_file=None, transcriptfile=None):
    R.start(transcriptfile, verbose=verbose)
    # Load data from corresponding tab file
    tabfilename = make_tabfilename(outdir, namebase)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    R.plot("save.distances", outdir, namebase, tabfilename,
           mnemonic_file=mnemonic_file, clobber=clobber, ropts=ropts)

def save_html(dirpath, feature_file, mnemonic_file=None,
              clobber=False, verbose=True):
    feature_base = os.path.basename(feature_file)

    title = "%s (%s)" % (HTML_TITLE_BASE, feature_base)

    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  verbose=verbose, title=title, mnemonic_file=mnemonic_file,
                  module=MODULE, featurefilename=feature_base)

def calc_distance_bin(n_bins, bin_width, distance, stranded):
    if distance == 0:
        return 0

    sign = (distance > 0) * 2 - 1
    bin = int((distance - sign) / bin_width) + sign
    limit = n_bins * bin_width
    return min(max(bin, -n_bins - 1), n_bins + 1)

def test_calc_distance_bin():
    params = [(10, 5),
              (5, 13),
              (1, 2)]

    for n_bins, bin_width in params:
        stranded = True
        bins = range(n_bins * 2 + 3)
        tests = [(0, 0),
                 (1, 1),
                 (bin_width, 1),
                 (bin_width + 1, 2),
                 (-1, bins[-1]),
                 (-2, bins[-1]),
                 (n_bins * bin_width, n_bins),
                 (n_bins * bin_width + 1, n_bins + 1),
                 (-n_bins * bin_width - 1, n_bins + 2),
                 (-n_bins * bin_width, n_bins + 3)]
        for distance, answer in tests:
            index = calc_distance_bin(n_bins, bin_width, distance, stranded)
            assert bins[index] == answer, """
Failed test:
  bins:      %r
  bin width: %r
  distance:  %r
  found:     %r (%r)
  correct:   %r
""" % (n_bins, bin_width, distance, bins[index], index, answer)

def print_distances(segmentation, annotation,
                    n_bins=DEFAULT_BINS,
                    bin_width=DEFAULT_BIN_WIDTH,
                    verbose=True,
                    stranded=False,
                    print_nearest=False,
                    all_overlapping=False):
    segment_data = segmentation.chromosomes
    feature_data = annotation.chromosomes

    labels = segmentation.labels
    groups = annotation.labels

    print DELIM.join(PRINT_FIELDS)

    bin_distances = range(0, n_bins * bin_width + 1, bin_width) + \
        ["Inf"]
    if stranded:
        bin_distances.extend(["-Inf"] + range(-n_bins * bin_width, 0, bin_width))

    log("Calulating distances...", verbose)
    start_time = time()
    for chrom in segment_data:
        segments = segment_data[chrom]
        try:
            features = feature_data[chrom]
        except KeyError:
            continue

        log("  %s" % chrom, verbose)
        feature_scanner = FeatureScanner(features)

        count_len = n_bins * (stranded + 1) + 2 + stranded
        assert count_len == len(bin_distances)
        counts = zeros((len(labels), len(groups), count_len), dtype="int")
        for segment in segments:
            nearest = feature_scanner.nearest(segment)
            distance = NaN
            if nearest is not None:
                if isinstance(nearest, list):
                    # If we are only printing one, use the first regardless
                    if not all_overlapping:
                        nearest = nearest[0:1]
                else:
                    # Make sure nearest is a list, even if only one element
                    nearest = [nearest]

                # Save multiple distances as a list of them
                for feature in nearest:
                    distance = FeatureScanner.distance(segment, feature)
                    if stranded:
                        try:
                            strand = feature['strand']
                            if strand == ".":
                                strand = None
                        except IndexError:
                            strand = None

                        if strand is None:
                            die("Trying to use strand information but it was"
                                " not found")

                        if (strand == "+" and \
                                segment['end'] < feature['start']) or \
                                (strand == "-" and \
                                     segment['start'] > feature['end']):
                            distance = -distance


                    count_bin = \
                        calc_distance_bin(n_bins, bin_width,
                                          distance, stranded)

                    counts[segment['key'], feature['key'], count_bin] += 1

                    if print_nearest:
                        name = groups[feature['key']]
                        distance_str = "%s %s" % (distance, name)
                    else:
                        distance_str = str(distance)

                    print_line(labels, chrom, segment, distance_str)

    log("Distances calculated in %.1f seconds" % (time() - start_time), verbose)
    return counts, bin_distances

def feature_distance(segment_file, annotation_file,
                     outdir, mnemonic_file=None,
                     n_bins=DEFAULT_BINS, bin_width=DEFAULT_BIN_WIDTH,
                     clobber=False, ropts=None,
                     stranded=False, print_nearest=False,
                     all_overlapping=False,
                     verbose=True, replot=False, noplot=False):
    if not replot:
        setup_directory(outdir)
        segmentation = Segmentation(segment_file, verbose=verbose)
        annotation = Annotation(annotation_file, verbose=verbose)
        labels = segmentation.labels
        groups = annotation.labels
        counts, bins = print_distances(segmentation, annotation,
                                       n_bins=n_bins, bin_width=bin_width,
                                       verbose=verbose,
                                       stranded=stranded,
                                       print_nearest=print_nearest,
                                       all_overlapping=all_overlapping)
        save_tab(counts, bins, labels, groups, outdir,
                 clobber=clobber, verbose=verbose)

    if not noplot:
        with open_transcript(outdir, MODULE) as transcriptfile:
            save_plot(outdir, clobber=clobber, verbose=verbose,
                      mnemonic_file=mnemonic_file, ropts=ropts,
                      transcriptfile=transcriptfile)

    save_html(outdir, annotation_file, mnemonic_file=mnemonic_file,
              clobber=clobber, verbose=verbose)

def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] SEGMENTATION ANNOTATION"
    version = "%%prog %s" % __version__

    parser = OptionParser(usage=usage, version=version,
                          description=__doc__.strip())

    group = OptionGroup(parser, "Basic options")
    add_common_options(group, ['clobber', 'quiet'])
    group.add_option("-s", "--stranded", dest="stranded",
                     default=False, action="store_true",
                     help="Strand correct features in the ANNOTATION file."
                     " If the feature contains strand information,"
                     " the sign of the distance value is used to portray the"
                     " relationship between the segment and the nearest"
                     " stranded feature. A negative distance value"
                     " indicates that the segment is nearest the 5' end of the"
                     " feature, whereas a positive value indicates the"
                     " segment is nearest the 3' end of the feature.")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Print options")
    group.add_option("-p", "--print-nearest", dest="print_nearest",
                     default=False, action="store_true",
                     help="The name of the nearest feature will be printed"
                     " after each distance (with a space separating the"
                     " two) for features from the ANNOTATION file."
                     " If multiple features are equally near (or overlap),"
                     " it is undefined which will be printed")
    group.add_option("-a", "--all-overlapping", dest="all_overlapping",
                     default=False, action="store_true",
                     help="If multiple features in the ANNOTATION file"
                     " overlap a segment, a separate line is printed for each"
                     " overlapping segment-feature pair. This is most"
                     " interesting with --print-nearest. Otherwise,"
                     " the first overlapping segment will be used for"
                     " printing.")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Plotting options")
    add_common_options(group, ['noplot', 'replot'])
    group.add_option("-n", "--n-bins", metavar="N", type="int",
                     dest="n_bins", default=DEFAULT_BINS,
                     help="Number of bins to use in histogram for distances"
                     " greater than zero and less than or equal to N*W."
                     " Distances of 0 and greater than N*W are placed in"
                     " additional bins. If --stranded, N more"
                     " bins are included in the negative direction and"
                     " a bin for distances less than -N*W.")
    group.add_option("-w", "--bin-width", metavar="W", type="int",
                     dest="bin_width", default=DEFAULT_BIN_WIDTH,
                     help="Number of bases in each bin."
                     " If --stranded, bins cover distances of"
                     " (-Inf,-N*W), ..., [-W,0), [0], (0,W], ..., (N*W,Inf)."
                     " Otherwise, the bins cover distances of"
                     " [0], (0,W], (W,2W], ..., (N*W,Inf).")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Output options")
    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) < 2:
        parser.error("Insufficient number of arguments")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)

    kwargs = dict(options.__dict__)

    feature_distance(*args, **kwargs)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 339 $"

"""
Provides command-line and package entry points for analyzing nucleotide
and dinucleotide frequencies for each label in the segmentation or annotation
file.

"""

import numpy
import os
import sys
import warnings

from collections import defaultdict
from genomedata import Genome
from numpy import zeros, bincount, array

from . import log, Annotation, die, RInterface, open_transcript, \
     add_common_options
from .common import make_tabfilename, setup_directory, tab_saver

from .html import save_html_div

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE="nucleotide_frequency"

HTML_TITLE = "Nucleotide and dinucleotide content"
HTML_TEMPLATE_FILENAME = "nucleotide_div.tmpl"

# Fieldnames must agree with categories in order and content
# with a "label" category at the beginning and nuc before dinuc
FIELDNAMES = ["label", "A|T", "C|G", "?", "AA|TT", "AC|GT",
              "AG|CT", "AT", "CA|TG", "CC|GG", "CG", "GA|TC", "GC", "TA",
              "??"]
NAMEBASE = "%s" % MODULE

NUC_CATEGORIES = [('A','T'), ('C', 'G')]
DINUC_CATEGORIES = [[('A', 'A'), ('T', 'T')],
                    [('A', 'C'), ('G', 'T')],
                    [('A', 'G'), ('C', 'T')],
                    [('A', 'T')],
                    [('C', 'A'), ('T', 'G')],
                    [('C', 'C'), ('G', 'G')],
                    [('C', 'G')],
                    [('G', 'A'), ('T', 'C')],
                    [('G', 'C')],
                    [('T', 'A')]]

R = RInterface(["common.R", "dinucleotide.R"])

def reduce_nucs(arr):
    # Return array with nucs compressed to 0-4
    return numpy.digitize(arr, [ord(i) for i in 'CGNT'])

def zip_nucs(nuc1, nuc2):
    # Given 2 reduced nuc arrays, return the dinuc value where dinucs are
    # formed from zip(nuc1, nuc2)
    # each reduced nuc array has a range of 0-4
    return nuc1 * 5 + nuc2

## Caclulates nucleotide and dinucleotide frequencies over the specified
## annotation data
def calc_nucleotide_frequencies(annotation, genome,
                                nuc_categories=NUC_CATEGORIES,
                                dinuc_categories=DINUC_CATEGORIES,
                                quick=False, verbose=True):

    # Store categories efficiently as dict
    # from each entry directly to category index
    quick_nuc_categories = defaultdict(dict)
    for index, category in enumerate(nuc_categories):
        for entry in category:
            key = reduce_nucs(array([ord(entry)], dtype='i8'))[0]
            #print("Mapping %s -> %d -> %d" % (entry, key, index))
            assert key not in quick_nuc_categories, \
                   "Error: nucleotide hash for %s is not unique" % entry
            quick_nuc_categories[key] = index
    quick_nuc_categories = dict(quick_nuc_categories)

    quick_dinuc_categories = defaultdict(dict)
    for index, category in enumerate(dinuc_categories):
        for entry in category:
            # Second letter between 0-4, first letter (0-4)*5
            nucs = reduce_nucs(array([ord(i) for i in entry], dtype='i8'))
            key = zip_nucs(nucs[0], nucs[1])
            #print("Mapping %s%s -> %d -> %d" % (entry[0], entry[1], key, index))
            assert key not in quick_dinuc_categories, \
                   "Error: dinucleotide hash for %s%s is not unique" % \
                   (entry[0], entry[1])
            quick_dinuc_categories[key] = index
    quick_dinuc_categories = dict(quick_dinuc_categories)

    # Store counts of each (di)nucleotide observed
    # separated by label
    nuc_counts = defaultdict(dict)
    dinuc_counts = defaultdict(dict)

    labels = annotation.labels
    for label_key in labels.iterkeys():
        nuc_counts[label_key] = zeros(len(nuc_categories)+1, dtype=numpy.long)
        dinuc_counts[label_key] = zeros(len(dinuc_categories)+1,
                                        dtype=numpy.long)

    # Count (di)nucleotides over annotation
    for chromosome in genome:
        chrom = chromosome.name

        try:
            chrom_rows = annotation.chromosomes[chrom]
            # Store entire chromosome's sequence as string in memory for speed
            warnings.simplefilter("ignore")
            log("  %s" % chrom, verbose)
            log("    preparing chromosome sequence...", verbose)
            chrom_sequence = chromosome.seq[0:chromosome.end]
            # Convert to capital letter within integer representation
            chrom_sequence[chrom_sequence >= ord('a')] -= ord('a') - ord('A')
            # Convert to reduced integer representation (0-4)
            chrom_sequence = reduce_nucs(chrom_sequence)
            warnings.resetwarnings()
        except KeyError:
            continue

        log("    counting nucs and dinucs...", verbose)
        interval = int(len(chrom_rows) / 70)
        row_i = 0
        for row in chrom_rows:
            if row_i % interval == 0:
                n_done = int(row_i / interval)
                n_left = 70 - n_done
                log("    %s%s" % ('=' * n_done, '-' * n_left), verbose)

            start = row['start']
            end = row['end']
            label = row['key']

            lab_nuc_counts = nuc_counts[label]
            lab_dinuc_counts = dinuc_counts[label]
            sequence = chrom_sequence[start:end]
            # Count nucs
            val_counts = bincount(sequence)
            for val in val_counts.nonzero()[0]:
                n = val_counts[val]
                # Put in last bin if not found
                index = quick_nuc_categories.get(val, len(lab_nuc_counts) - 1)
                lab_nuc_counts[index] += n

            # Count dinucs efficiently
            if len(sequence) > 1:
                val_counts = bincount(zip_nucs(sequence[:-1], sequence[1:]))
                for val in val_counts.nonzero()[0]:
                    n = val_counts[val]
                    index = quick_dinuc_categories.get(val, len(lab_dinuc_counts) - 1)
                    dinuc_counts[label][index] += n

            row_i += 1

        # Only look at first chromosome if quick
        if quick: break

    return (nuc_counts, dinuc_counts)


def make_row(label, nuc_counts, dinuc_counts, fieldnames=FIELDNAMES):
    row = {}

    for i, field in enumerate(fieldnames):
        if field == "label":
            row[field] = label
        elif i <= len(nuc_counts):
            # Compensate for "label" field
            row[field] = nuc_counts[i - 1]
        else:
            # Compensate for "label" and nucleotide fields
            row[field] = dinuc_counts[i - len(nuc_counts) - 1]
    return row

# Fieldnames must agree with categories in order and content
# with a "label" category at the beginning, and nuc before dinuc
def save_tab(labels, nuc_counts, dinuc_counts, dirpath,
             fieldnames=FIELDNAMES, clobber=False, verbose=True):
    with tab_saver(dirpath, NAMEBASE, fieldnames,
                   clobber=clobber, verbose=verbose) as saver:
        for label_key in sorted(labels.keys()):
            row = make_row(labels[label_key],
                           nuc_counts[label_key],
                           dinuc_counts[label_key])
            saver.writerow(row)

def save_plot(dirpath, clobber=False, verbose=True, ropts=None,
              mnemonic_file="", namebase=NAMEBASE, transcriptfile=None):
    R.start(verbose=verbose, transcriptfile=transcriptfile)

    tabfilename = make_tabfilename(dirpath, NAMEBASE)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    R.plot("save.dinuc", dirpath, namebase, tabfilename,
           mnemonic_file=mnemonic_file, clobber=clobber, ropts=ropts)

def save_html(dirpath, clobber=False, verbose=True, mnemonicfile=None):
    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  tables={"":NAMEBASE}, module=MODULE, verbose=verbose,
                  mnemonicfile=mnemonicfile, title=HTML_TITLE)

## Package entry point
def validate(filename, genomedatadir, dirpath, clobber=False, quick=False,
             replot=False, noplot=False, mnemonic_file=None, verbose=True,
             ropts=None):
    setup_directory(dirpath)
    if not replot:
        annotation = Annotation(filename, verbose=verbose)
        labels = annotation.labels

        with Genome(genomedatadir) as genome:
            nuc_counts, dinuc_counts = \
                calc_nucleotide_frequencies(annotation, genome,
                                            quick=quick, verbose=verbose)

        save_tab(labels, nuc_counts, dinuc_counts, dirpath,
                 clobber=clobber, verbose=verbose)

    if not noplot:
        with open_transcript(dirpath, MODULE) as transcriptfile:
            save_plot(dirpath, clobber=clobber, verbose=verbose,
                      mnemonic_file=mnemonic_file, ropts=ropts,
                      transcriptfile=transcriptfile)

    save_html(dirpath, clobber=clobber, mnemonicfile=mnemonic_file,
              verbose=verbose)

def parse_options(args):
    from optparse import OptionGroup, OptionParser

    usage = "%prog [OPTIONS] ANNOTATION GENOMEDATAFILE"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    group = OptionGroup(parser, "Flags")
    add_common_options(group, ['clobber', 'quiet', 'quick', 'replot',
                               'noplot'])
    parser.add_option_group(group)

    group = OptionGroup(parser, "Output")
    add_common_options(group, ['mnemonic_file', 'outdir'], MODULE=MODULE)
    parser.add_option_group(group)

    group = OptionGroup(parser, "R options")
    add_common_options(group, ['ropts'])
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) != 2:
        parser.error("Inappropriate number of arguments")

    return (options, args)

## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    filename = args[0]
    genomedatadir = args[1]

    kwargs = dict(options.__dict__)
    outdir = kwargs.pop("outdir")
    validate(filename, genomedatadir, outdir, **kwargs)

if __name__ == "__main__":
    sys.exit(main())
